#pragma once

#include <iostream>
#include <iomanip>
#include <fstream>
#include <ctime>
#include <vector>
#include <algorithm>
#include <iterator>
#include <string>
#include <cmath>

void createDataFileStreams(const std::string& file) {

	srand(time(nullptr));


	std::ofstream f(file, std::ios::out | std::ios::trunc);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	for (int i = 0; i < 100; ++i) {
		f << rand() % 101 - 50 << "\n";
	}
	f.close();
}

template<class container>
void createDataGenerateCopy(container& cont, const std::string& file) {
	srand(time(nullptr));

	std::ofstream f(file, std::ios::out | std::ios::trunc);
	std::vector<container::value_type> values(100);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	std::generate(values.begin(), values.end(), []() {
		return (std::rand() % 101 - 50);
		});
	std::copy(values.begin(), values.end(), std::ostream_iterator<container::value_type>(f, "\n"));
	f.close();
}

template<class container>
container loadDataFileStream(const std::string& file) {
	container res;
	std::ifstream f(file);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	typename container::value_type buf;
	while (f >> buf) {
		res.push_back(buf);
	}
	f.close();
	return res;
}

template<class container>
container loadDataIterCopy(const std::string& file) {
	container res;
	std::ifstream f(file);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	std::copy(std::istream_iterator<container::value_type>(f), std::istream_iterator<container::value_type>(), res.begin());
	return res;
}

template<class container>
void modifyCont(container& cont) {
	typename container::value_type sum = 0;
	for (const auto& i : cont) {
		sum += std::abs(i);
	}
	typename container::value_type mean = sum / 100;
	for (auto& i : cont) {
		if (cont[i] == 0) {
			cont[i] = mean;
		}
	}
	cont.push_back(sum);
	cont.push_back(mean);
}

template<class iterator>
void modifyIters(iterator begin, iterator end) {
	std::vector<iterator::value_type> values(begin, end);
	typename iterator::value_type sum = 0;
	for (auto iter = begin; iter != end; ++iter) {
		sum += std::abs(*iter);
	}
	typename iterator::value_type mean = sum / 100;
	for (auto& iter = begin; iter != end; ++iter) {
		if (*iter == 0) {
			*iter = mean;
		}
	}
}

template<class container>
void modifyForEach(container& cont) {
	typename container::value_type sum = 0;
	std::for_each(cont.begin(), cont.end(), [&sum](const auto& obj) {
		sum += std::abs(obj);
		});
	typename container::value_type mean = sum / 100;
	std::for_each(cont.begin(), cont.end(), [&sum](auto& obj) {
		if (obj == 0) {
			obj = mean;
		}
		});
}

template<class container>
void modifyTransform(container& cont, const std::string& file) {
	std::vector<container::value_type> buf(100);
	std::ifstream fin(file);
	if (!fin.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	std::copy(std::istream_iterator<container::value_type>(fin), std::istream_iterator<container::value_type>(), buf.begin());
	typename container::value_type sum = 0;
	std::for_each(buf.begin(), buf.end(), [&sum](const auto& obj) {
		sum += std::abs(obj);
		});
	typename container::value_type mean = sum / 100;
	std::ofstream fout("result.txt");
	std::transform(buf.begin(), buf.end(), std::ostream_iterator<container::value_type>(fout, "\n"), [&sum](auto& obj) {
		return obj = mean;
		});
}

template<class container>
void outputResultFileStream(const container& cont, const std::string& file) {
	std::ofstream f(file, std::ios::out | std::ios::app);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	auto iter = cont.begin();
	while (iter != cont.end()) {
		f << *iter << "\n";
		++iter;
	}
	f.close();
}

template<class container>
void outputResultCopyIter(const container& cont, const std::string& file) {
	std::ofstream f(file, std::ios::out | std::ios::trunc);
	if (!f.is_open()) {
		throw std::exception("FILE NOT EXIST");
	}
	auto iter = cont.begin();
	std::copy(cont.begin(), cont.end(), std::ostream_iterator<container::value_type>(f, "\n"));
}
