#include "reg.h"


void toTxt(const std::map<size_t, Products>& prod, const std::string& resultFileName)
{
    
	std::ofstream text(resultFileName);
	if (!text.is_open())
	{
		std::cout << "File error";
	} 
	text << 
		std::setw(14) << "Storage Id" << std::setw(6) << "|" <<
		std::setw(14) << "Product Code" << std::setw(6) << "|" <<
		std::setw(14) << "Product Name" << std::setw(6) << "|" <<
		std::setw(14) << "Arrival Date" << std::setw(6) << "|" <<
		std::setw(14) << "Shelf Life" << std::setw(6) << "|" <<
		std::setw(14) << "Products Amount" << std::setw(6) << "|" <<
		std::setw(14) << "Unit Price" << std::setw(6) << "|";
	for (const auto& p : prod)
	{
		text << p.second << "\n";
	}
	text.close();
}

void Reg::newProduct(Products& prod)
{
	std::cout << "Input data of the new product";
	std::cin >> prod;
}

void Reg::load()
{
	std::ifstream in_f(binName, std::ios::binary | std::ios::out | std::ios::in); 
	if (!in_f.is_open())
	{
		std::cout << "File error";
	}
	else
	{
		Products tmp_prod;
		while (in_f.read(reinterpret_cast<char*>(&tmp_prod), sizeof(tmp_prod)))
		{
			amount++;
			product.insert(std::make_pair(amount, tmp_prod));
		}
		in_f.close();
	}
}

Reg::Reg(const std::string& bin_f, const std::string& txt_to_out) : binName(bin_f), txtName(txt_to_out)
{
	load();
}

Reg::~Reg()
{
	product.clear();
	binName.clear();
	txtName.clear();
}

void Reg::createAndWrite()
{
	if (!product.empty())
	{
		amount = 0;
		product.clear();
		std::fstream clear(binName, std::ios::binary | std::ios::out | std::ios::in);
		clear.close();
	}
	std::ofstream new_b_file(binName, std::ios::binary | std::ios::out);
	if (!new_b_file.is_open())
	{
		std::cout << "File error";
	}
	else
	{
		char choice;
		while (choice != 'n')
		{
			std::cout << "Want to add new data? y/n";
			std::cin >> choice;
			if (choice == 'n')
			{
				break;
			}
			else if (choice == 'y')
			{
				amount++;
				Products newProd;
				newProduct(newProd);
				new_b_file.write(reinterpret_cast<char*>(&newProd), sizeof(newProd));
				product.insert(std::move(std::make_pair(amount, newProd)));
			}
		}
		new_b_file.close();
	}
}

void Reg::printProduct(const std::pair<size_t, Products>& prod)
{
	std::cout << "Serial number" << prod.first;
	std::cout << "Products" << prod.second;

}

void Reg::updateFile()
{
	std::ofstream curr_file(binName, std::ios::binary | std::ios::out);
	if (!curr_file.is_open())
	{
		std::cout << "File error";
	}
	for (auto& p : product)
	{
		curr_file.write(reinterpret_cast<char*>(&p.second), sizeof(Products));

	}
	curr_file.close();
}

void Reg::printData()  noexcept
{
	std::cout << "Data from the" << binName << std::endl;
	for (auto& el : product)
	{
		printProduct(el);
	}
}

void Reg::addProduct()
{
	std::ofstream curr_File(binName, std::ios::binary | std::ios::app);
	if (!curr_File.is_open())
	{
		std::cout << "File error";
	}
	else
	{
		Products newprod;
		amount++;
		newProduct(newprod);
		product.insert(std::make_pair(amount, newprod));
		curr_File.write(reinterpret_cast<char*>(&newprod), sizeof(Products));
		curr_File.close();
	}
}

void Reg::dropProduct(int prod_id)
{
	if (prod_id <= 0 || prod_id > int(amount))
	{
		std::cout << "Error id";
	}
	product.erase(prod_id);
	--amount;
	for (int i = prod_id + 1; i <= int(amount) + 1; i++)
	{
		auto tmp_iter = product.at(i);
		product.erase(i);
		product.emplace(std::make_pair(i - 1, tmp_iter));
	}
	updateFile();
}


void Reg::changeProduct(int prod_id)
{
	auto update_iterator = std::find_if(product.begin(), product.end(), [&prod_id](auto p) {return p.first == prod_id; });
	newProduct(update_iterator->second);
	updateFile();
}

void Reg::findStorageId(int id) noexcept
{
	std::map<size_t, Products> result_map;
	std::map<size_t, Products>::iterator current_iter = product.begin();
	while (current_iter != product.end())
	{
		current_iter = find_if(product.begin(), product.end(), [&id](const std::pair<const size_t, Products>& tmpProd)
		{return tmpProd.second.storageId == id;  });
		if (current_iter != product.end())
		{
			result_map.insert(*current_iter);
			current_iter++;
		}
	}
	if (result_map.empty())
	{
		std::cout << "Storage empty";
	}
	else
	{
		toTxt(result_map, "result.txt");
	}
}

void Reg::findProductCode(int code) noexcept
{
	std::map<size_t, Products> result_map;
	std::map<size_t, Products>::iterator current_iter = product.begin();
	while (current_iter != product.end())
	{
		current_iter = find_if(product.begin(), product.end(), [&code](const std::pair<const size_t, Products>& tmpProd)
		{return tmpProd.second.productCode == code;  });
		if (current_iter != product.end())
		{
			result_map.insert(*current_iter);
			current_iter++;
		}
	}
	if (result_map.empty())
	{
		std::cout << "Error code";
	}
	else
	{
		toTxt(result_map, "result.txt");
	}
}

void Reg::findArrivalDate(const Date& arr_date) noexcept
{
	std::map<size_t, Products> result_map;
	std::map<size_t, Products>::iterator current_iter = product.begin();
	while (current_iter != product.end())
	{
		current_iter = find_if(product.begin(), product.end(), [&arr_date](const std::pair<const size_t, Products>& tmpProd)
		{return tmpProd.second.arrivalDate == arr_date;  });
		if (current_iter != product.end())
		{
			result_map.insert(*current_iter);
			current_iter++;
		}
	}
	if (result_map.empty())
	{
		std::cout << "Error arrivcal date";
	}
	else
	{
		toTxt(result_map, "result.txt");
	}
}

void Reg::findOverdueProducts(const Date& curr_date) noexcept
{
	std::map<size_t, Products> result_map;
	std::map<size_t, Products>::iterator current_iter = product.begin();
	while (current_iter != product.end())
	{
		current_iter = find_if(product.begin(), product.end(), [&curr_date](const std::pair<const size_t, Products>& tmpProd) 
		{
			return !((tmpProd.second.arrivalDate + tmpProd.second.shelfLife) < curr_date); 
			});
		if (current_iter != product.end())
		{
			result_map.insert(*current_iter);
			current_iter++;
		}
	}
	if (result_map.empty())
	{
		std::cout << "Error overdue products";
	}
	else
	{
		toTxt(result_map, "result.txt");
	}
}

void Reg::findNotOverdueProducts(const Date& curr_date) noexcept
{
	std::map<size_t, Products> result_map;
	std::map<size_t, Products>::iterator current_iter = product.begin();
	while (current_iter != product.end())
	{
		current_iter = find_if(product.begin(), product.end(), [&curr_date](const std::pair<const size_t, Products>& tmpProd) 
		{
			return (tmpProd.second.arrivalDate + tmpProd.second.shelfLife) < curr_date;  
			});
		if (current_iter != product.end())
		{
			result_map.insert(*current_iter);
			current_iter++;
		}
	}
	if (result_map.empty())
	{
		std::cout << "Error not overdue products";
	}
	else
	{
		toTxt(result_map, "result.txt");
	}
}

void Reg::loadOutputTxtFile()
{
	std::ofstream text(txtName);
	double all_summ = 0.;
	for (auto& p : product)
	{
		text << p.first << ". " << "\n";
		text << p.second;
		all_summ += (p.second.productsAmount * p.second.unitPrice);
	}
	text << "Journal:" << amount << "\n" << "Total price: " << all_summ;
	text.close();
}

